/**
* ValidateSelfCert : validates an X.509 certification path
*      using a PKIX CertPathValidator
*
* Synopsis: java ValidateSelfCert keystore password alias
*
* The program validates the certificate chain in the keystore
*  referred by the "alias".
*/
import java.security.*;
import java.security.cert.*;
import java.util.*;
import java.io.*;

class ValidateSelfCert
{
    public static void main(String args[])
    {
        // Instantiate a KeyStore with type JKS
        try {
            if (args.length != 3)
                throw new Exception("ValidateCert " + "keystore " + "password " + "alias");

           KeyStore ks = KeyStore.getInstance("JKS");

            // Load the contents of the KeyStore
            ks.load(new FileInputStream(args[0]),
                args[1].toCharArray());

             // Fetch certificate chain stored with alias "rags"
             java.security.cert.Certificate[] certArray = ks.getCertificateChain(args[2]);

             if (certArray == null)
                throw new Exception("Alias " + args[2] + " is not a certificate chain");

             // Convert chain to a List
             List certList = Arrays.asList(certArray);

             // Instantiate a CertificateFactory for X.509
             CertificateFactory cf = CertificateFactory.getInstance("X.509");

             // Extract the certification path from
             // the List of Certificates
             CertPath cp = cf.generateCertPath(certList);

             // Create CertPathValidator that implements the "PKIX" algorithm
             CertPathValidator cpv = null;
             cpv = CertPathValidator.getInstance("PKIX");

             // Set the Trust anchor
             TrustAnchor anchor = new TrustAnchor((X509Certificate)ks.getCertificate(args[2]), null);

             // Set the PKIX parameters
             PKIXParameters params = new PKIXParameters(Collections.singleton(anchor));
             params.setRevocationEnabled(false);

             // Validate and obtain results
             try {
                 PKIXCertPathValidatorResult result =
                     (PKIXCertPathValidatorResult) cpv.validate(cp, params);
                 PolicyNode policyTree = result.getPolicyTree();
                 PublicKey subjectPublicKey = result.getPublicKey();

                 System.out.println("Certificate validated");
                 System.out.println("Policy Tree:\n" + policyTree);
                 System.out.println("Subject Public key:\n" +subjectPublicKey);

             } catch (CertPathValidatorException cpve) {
                 System.out.println("Validation failure, cert[" 
                     + cpve.getIndex() + "] :" + cpve.getMessage());
             }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}